searchData={"items":[{"type":"module","title":"Flashy.Component","doc":"Struct containing a notification module and function for being used later with `apply/3` function.","ref":"Flashy.Component.html"},{"type":"function","title":"Flashy.Component.new/1","doc":"","ref":"Flashy.Component.html#new/1"},{"type":"function","title":"Flashy.Component.new/2","doc":"","ref":"Flashy.Component.html#new/2"},{"type":"type","title":"Flashy.Component.t/0","doc":"","ref":"Flashy.Component.html#t:t/0"},{"type":"protocol","title":"Flashy.Protocol","doc":"Protocol used to fetch information about the notification component to be rendered.","ref":"Flashy.Protocol.html"},{"type":"function","title":"Flashy.Protocol.function_name/1","doc":"","ref":"Flashy.Protocol.html#function_name/1"},{"type":"function","title":"Flashy.Protocol.module/1","doc":"","ref":"Flashy.Protocol.html#module/1"},{"type":"type","title":"Flashy.Protocol.t/0","doc":"All the types that implement this protocol.","ref":"Flashy.Protocol.html#t:t/0"},{"type":"extras","title":"Flashy","doc":"# Flashy\n\n[![Hex](https://img.shields.io/hexpm/v/flashy.svg)](https://hex.pm/packages/flashy)\n[![Hexdocs](https://img.shields.io/badge/-docs-green)](https://hexdocs.pm/flashy)\n\nFlashy is a small library that extends LiveView's flash support to function and live components.\n\n[2023-10-22 20-43-38.webm](https://github.com/sezaru/flashy/assets/279828/ac1784ab-1126-4625-8097-81d8fc40adb1)","ref":"readme.html"},{"type":"extras","title":"Installation - Flashy","doc":"First add `Flashy` to your list of dependencies in `mix.exs`:\n\n``` elixir\ndef deps do\n  [\n    {:flashy, \"~> 0.2.4\"}\n  ]\nend\n```\n\nNow, inside `assets/js/app.js`, add `flashy` hooks:\n\n``` javascript\nimport FlashyHooks from \"flashy\"\n\n// if you don't have any other hooks:\nlet liveSocket = new LiveSocket(\"/live\", Socket, {params: {_csrf_token: csrfToken}, hooks: FlashyHooks})\n\n// if you have other hooks:\nconst hooks = {\n    MyHook: {\n        // ...\n    },\n    ...FlashyHooks\n}\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {params: {_csrf_token: csrfToken}, hooks})\n```\n\nNow, inside `assets/tailwind.config.js`:\n\n``` javascript\n...\n\nmodule.exports = {\n  content: [\n    \"./js/**/*.js\",\n    \"../lib/flashy_example_web.ex\",\n    \"../lib/flashy_example_web/**/*.*ex\",\n    \"../deps/flashy/**/*.*ex\", //  _web.ex` and add the following to `html_helpers` function:\n\n``` elixir\n  defp html_helpers do\n    quote do\n      ...\n\n      # Add Flash notifications functionality\n      import Flashy\n    end\n  end\n```\n\nFinally, you need to update your `lib/ _web/components/layouts/app.html.heex`:\n\nReplace the line:\n\n``` elixir\n \n```\n\nWith:\n\n``` elixir\n \n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Flashy notification hook - Flashy","doc":"`Flashy` also has a hook that can be added to `LiveViews` to make it easier to send notifications from a `LiveComponent` to a `LiveView` when no navigation is happening.\n\nFirst, we need to add the hook to a `LiveView` `live_session`:\n\n``` elixir\nlive_session :my_session, on_mount: [Flashy.Hook] do\n  ...\nend\n```\n\nNow, inside any `LiveComponent`, we can send a notification to the `LiveView` like this:\n\n``` elixir\nnotification = Notification.Normal.new(:success, \"some notification\")\n\nsocket = send_notification(socket, notification)\n```\n\n#","ref":"readme.html#flashy-notification-hook"},{"type":"extras","title":"Using with controllers - Flashy","doc":"If you want to also use `Flashy` with controllers outside `LiveView`, first you need to go to your web file `lib/ _web.ex` and add the following to `controller` function:\n\n``` elixir\ndefp controller do\n  quote do\n    ...\n\n    # Add Flash notifications functionality\n    import Flashy\n  end\nend\n```\n\nNow, inside any `Controller`, we can send a notification like this:\n\n``` elixir\ndef index(conn, _) do\n  notification = Notifications.Normal.new(:info, \"some notification\")\n\n  conn |> put_notification(notification) |> render(:index)\nend\n```\n\n#","ref":"readme.html#using-with-controllers"},{"type":"extras","title":"Disconnected notifications - Flashy","doc":"Now we need to, at least, implement the disconnected notification. `Flashy` doesn't come with any pre-defined disconnected notification design, so you need to implement it yourself.\n\nHere is an example of one implementation using `PetalComponents` `alert` component:\n\n``` elixir\ndefmodule MyProjectWeb.Components.Notifications.Disconnected do\n  @moduledoc false\n\n  use MyProjectWeb, :html\n\n  use Flashy.Disconnected\n\n  import PetalComponents.Alert\n\n  attr :key, :string, required: true\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       \n        Attempting to reconnect  \n       \n     \n    \"\"\"\n  end\nend\n```\n\nNow you need to set the following config so `Flashy` knows what disconnected component we should use, to do that, in your `config/config.exs` add the following:\n\n``` elixir\nconfig :flashy,\n  disconnected_module: MyProjectWeb.Components.Notifications.Disconnected\n```\n\nNow we are all set, `Flashy` is ready to be used.","ref":"readme.html#disconnected-notifications"},{"type":"extras","title":"Adding normal notifications - Flashy","doc":"Above we setup a disconnected component which is mandatory, but you probably want to have a \"normal\" notification to use for simple messages.\n\n`Flashy` ships with a base implementation for a notification like that, it supports timed auto-hide with progress bar and showing or not a close button.\n\nTo implement it you just need to define how to render its body, similar to how we did with the disconnected component. Here is an example using `PetalComponents` `alert` component:\n\n``` elixir\ndefmodule MyProjectWeb.Components.Notifications.Normal do\n  @moduledoc false\n\n  use MyProjectWeb, :html\n\n  use Flashy.Normal, types: [:info, :success, :warning, :danger]\n\n  import PetalComponents.Alert\n\n  attr :key, :string, required: true\n  attr :notification, Flashy.Normal, required: true\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       \n           \n\n         \n       \n     \n    \"\"\"\n  end\n\n  attr :id, :string, required: true\n\n  defp progress_bar(assigns) do\n    ~H\"\"\"\n     \n    \"\"\"\n  end\n\n  defp color(type), do: to_string(type)\n\n  defp close_button_properties(%{closable?: true}, key),\n    do: [\"phx-click\": JS.exec(\"data-hide\", to: \"##{key}\")]\n\n  defp close_button_properties(%{closable?: false}, _), do: nil\nend\n```\n\nNote that you can set any `types` you want to the normal component, you just need to add it to the `types` list when calling `use Flashy.Normal`:\n\n``` elixir\nuse Flashy.Normal, types: [:info, :fatal, :some_other_type]\n```","ref":"readme.html#adding-normal-notifications"},{"type":"extras","title":"Adding a entirely custom notification - Flashy","doc":"You can also create 100% custom notifications for your needs, for example, `Flashy` supports live components when you need to store state or handle events, here I will show a custom notification that will how a form inside with a text input field.\n\nThe idea with this notification would be to allow you to create a notification with business logic, for example, if you are creating a chat application, you can have a notification that will allow users to reply to it directly from the notificatio itself.\n\nHere is the implementation:\n\n``` elixir\ndefmodule MyProjectWeb.Components.Notifications.Custom do\n  @moduledoc false\n\n  alias Flashy.{Component, Helpers}\n\n  use MyProjectWeb, :live_component\n\n  use TypedStruct\n\n  import PetalComponents.{Alert, Input, Button}\n\n  typedstruct enforce: true do\n    field :question, String.t()\n\n    field :target_module, module\n    field :target_id, String.t()\n\n    field :component, Component.t()\n  end\n\n  @spec new(String.t(), module, String.t()) :: t\n  def new(question, target_module, target_id) do\n    struct!(__MODULE__,\n      question: question,\n      target_module: target_module,\n      target_id: target_id,\n      component: Component.new(&live_render/1)\n    )\n  end\n\n  attr :key, :string, required: true\n  attr :notification, __MODULE__, required: true\n\n  attr :rest, :global\n\n  def live_render(%{key: key} = assigns) do\n    assigns = assign(assigns, id: key)\n\n    ~H\" \"\n  end\n\n  def update(assigns, socket) do\n    socket = socket |> assign(assigns) |> assign(form: to_form(%{}))\n\n    {:ok, socket}\n  end\n\n  def handle_event(\"send_answer\", %{\"answer\" => answer}, socket) do\n    %{id: id, notification: %{target_module: module, target_id: target_id}} = socket.assigns\n\n    send_update(module, id: target_id, answer: answer)\n\n    socket = push_event(socket, \"js-exec\", %{to: \"##{id}\", attr: \"data-hide\"})\n\n    {:noreply, socket}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       \n         \n           \n               \n\n             \n\n             \n           \n         \n       \n     \n    \"\"\"\n  end\nend\n\ndefimpl Flashy.Protocol, for: MyProjectWeb.Components.Notifications.Custom do\n  def module(notification), do: notification.component.module\n\n  def function_name(notification), do: notification.component.function_name\nend\n```\n\nThe main takeaway here is that you always need to generate a struct which implements the `Flashy.Protocol`, this is how `Flashy` know which component it needs to call to render.","ref":"readme.html#adding-a-entirely-custom-notification"},{"type":"extras","title":"Usage - Flashy","doc":"Now that we have `Flashy` installed with some notifications, to use it is pretty simple, here are some examples:\n\nShowing a `info` normal notification:\n\n``` elixir\nalias MyProjectWeb.Components.Notifications.Normal\n\nput_notification(socket, Normal.new(:info, \"My  cool  notification\"))\n```\n\nFlashy supports stacked notifications as-well, so you can do something like this:\n\n``` elixir\nalias MyProjectWeb.Components.Notifications.Normal\n\nsocket\n|> put_notification(Normal.new(:info, \"My  cool  notification\"))\n|> put_notification(Normal.new(:info, \"My another  cool  notification\"))\n|> put_notification(Normal.new(:danger, \"Fatal error notification\"))\n```\n\nWhen using normal notifications, you can also set if they are dimissable and how much time it will be visible:\n\n``` elixir\nalias MyProjectWeb.Components.Notifications.Normal\n\n# This option means the notification will never auto-hide, \n# the user will need to close it via the close button \noptions_1 = Flashy.Normal.Options.new(dismissible?: false)\n\n# This option means the notification will not show the close button\noptions_2 = Flashy.Normal.Options.new(closable?: false)\n\n# This option means you can set how much time the notification will show\n# before it auto-hides\noptions_3 = Flashy.Normal.Options.new(dismiss_time: :timer.seconds(2))\n\nsocket\n|> put_notification(Normal.new(:info, \"My  cool  notification\", options_1))\n|> put_notification(Normal.new(:info, \"My  cool  notification\", options_2))\n|> put_notification(Normal.new(:info, \"My  cool  notification\", options_3))\n```\n\nFinally, we, of course, can also create notifications with our own custom notifications:\n\n``` elixir\nalias MyProjectWeb.Components.Notifications.Custom\n\nput_notification(socket, Custom.new(\"How are you today?\", __MODULE__, id))\n```","ref":"readme.html#usage"},{"type":"extras","title":"More examples - Flashy","doc":"You can check how the library works by going to our [examples project](https://github.com/sezaru/flashy_example) to see it working in practice.","ref":"readme.html#more-examples"},{"type":"extras","title":"Customizing CSS - Flashy","doc":"By default `Flashy` notifications will show-up on the top right of the screen. But sometimes your requirements can be different, maybe you want the notifications to show-up on the left side, maybe the way `Flashy` default CSS doesn't work well with your project CSS, etc.\n\n`Flashy` uses `PhxComponentHelpers` to customize CSS, you can check the library here: https://hexdocs.pm/phx_component_helpers/PhxComponentHelpers.html\n\n`Flashy` allows to fully customize the CSS, as an example, I will show how to move the notifications to the left.\n\nThe first place we can change CSS is to the `Flashy.Container` component, you can update your `lib/ _web/components/layouts/app.html.heex` file to:\n\n``` elixir\n \n```\n\nWith this change, we are replacing `right-0` with `left-0` and `items-end` with `items-start`.\n\nNow, we want to customize our components, first, we will want to create a custom transition that will move from the left to right, let's create a module to add that since they are used by all notification components:\n\n``` elixir\ndefmodule MyProjectWeb.Components.Notifications.Helpers do\n  @moduledoc false\n\n  alias Phoenix.LiveView.JS\n\n  def hide_notification(key) do\n    JS.hide(\n      to: \"##{key}\",\n      transition: {\"ease-in duration-300\", \"translate-x-0\", \"translate-x-[-100%]\"},\n      time: 300\n    )\n    |> JS.push(\"lv:clear-flash\", value: %{key: key})\n  end\n\n  def show_notification(key) do\n    JS.show(\n      to: \"##{key}\",\n      transition: {\"ease-in duration-300\", \"translate-x-[-100%]\", \"translate-x-0\"},\n      time: 300\n    )\n  end\nend\n```\n\nNow let's start changing our components, let's start with the `Disconnected` one.\n\nFirst we add the alias to our new helper:\n\n``` elixir\nalias MyProjectWeb.Components.Notifications.Helpers\n```\n\nThen, inside the `render` function, we change the way we call `Flashy` disconnected render:\n\n``` elixir\n \n```\n\nWhat we are doing here is customize the CSS and the JS actions.\n\nFor the CSS, we replaced `!pr-3` with `pl-3` and `translate-x-full` with `translate-x-[-100$]`.\n\nFor the JS actions, we are using the ones for our helper instead of `Flashy` built-in ones.\n\nNow, let's do the same for the `Normal` component:\n\n``` elixir\nalias FlashyExampleWeb.Components.Notifications.Helpers\n\n...\n\n \n```\n\nIt is exactly the same changes are the `Disconnected` component above.\n\nFinally, we will also update our `Custom` component.\n\nOn that one we are importing `Flashy` built-in helpers, se we will replace that with ours:\n\n``` elixir\nalias FlashyExampleWeb.Components.Notifications.Helpers\n\nalias Flashy.Component\n```\n\nNow we just need to update the component CSS as-well.\n\nIn this case we are not using `PhxComponentHelpers`, so we will just implement the full class directly:\n\n``` elixir\n \n```\n\nAfter these changes, your notifications will show up on the left:\n\n[2023-10-23 20-23-15.webm](https://github.com/sezaru/flashy/assets/279828/94e3f31e-a932-42cd-967c-ccedef53dfcf)","ref":"readme.html#customizing-css"}],"content_type":"text/markdown"}